<?php

namespace App\Controllers\Cobrancas;

use App\Controllers\BaseController;
use App\Enums\StatusEnum;
use App\Enums\TipoLiquidacaoEnum;
use App\Libraries\ApiResponse;
use App\Libraries\Data;
use App\Models\CobrancasModel;
use App\Models\MetodosPagamentoModel;
use ReflectionException;

class Cobrancas extends BaseController
{
    public function index(): string
    {
        return $this->render();
    }

    public function webbook()
    {
        $segmentos = $this->request->getUri()->getSegments();
        $resposta = ApiResponse::init();

        if (!isset($segmentos[4]) || !is_numeric($segmentos[5])) {
            $resposta->error(self::$messageError);
        }

        $metodoPagamentoId = $segmentos[4];
        $cobrancaId = $segmentos[5];

        $metodosPagamentoModel = new MetodosPagamentoModel();
        $metodosPagamento = $metodosPagamentoModel->where([
            'id' => $metodoPagamentoId,
            'status' => StatusEnum::ATIVO->value
        ])->first();

        if (!$metodosPagamento) {
            $resposta->error('Método não encontrado!');
        }

        $cobrancasModel = new CobrancasModel();
        $cobrancas = $cobrancasModel->where([
            'id' => $cobrancaId,
            'status != ' => StatusEnum::PAGO->value
        ])->first();

        if (!$cobrancas) {
            $resposta->error('Cobrança não encontrada!');
        }

        if ($metodosPagamento->integrador === "FAKE") {
            try {
                $cobrancas->status = StatusEnum::PAGO->value;
                $cobrancas->tipo_liquidado = TipoLiquidacaoEnum::WEBHOOK->value;
                $cobrancas->data_pagamento = Data::agora(Data::FORMAT_DB_DATE);
                $cobrancas->data_liquidado = Data::agora(Data::FORMAT_DB_DATE);
                $cobrancasModel->save($cobrancas);
            } catch (ReflectionException $e) {
                $resposta->error($e->getMessage());
            }
        }

        $resposta->success('OK', $cobrancas);
    }
}