<?php

namespace App\Controllers\Dashboard;

use App\Controllers\BaseController;
use App\Enums\StatusEnum;
use App\Libraries\ApiResponse;
use App\Libraries\Arquivo;
use App\Libraries\Auth;
use App\Libraries\ExcecaoCertificaSe;
use App\Libraries\GerarCertificado;
use App\Libraries\Suporte;
use App\Models\CertificadosArquivosImportadosModel;
use App\Models\CertificadosInscritosModel;
use App\Models\CertificadosModel;
use App\Models\CertificadosModelosModel;
use Config\Database;
use League\Csv\Exception;
use League\Csv\InvalidArgument;
use League\Csv\Reader;
use League\Csv\Statement;
use ReflectionException;

class Certificados extends BaseController
{
    const NAO_EXIBIR_COLUNA_NO_VINCULO = ['codigo_validacao'];

    public function index()
    {
        return $this->render();
    }

    public function novo()
    {
        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'texto_introdutorio' => 'required|max_length[255]',
                'nome' => 'required|max_length[255]',
                'descricao' => 'required',
                'local_emissao' => 'required',
                'data_emissao' => 'required',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            $data = $this->validator->getValidated();
            $data['clientes_id'] = Auth::get('id');
            try {
                $model = new CertificadosModel();
                $model->save($data);
                $id = $model->getInsertID();
                $resposta->success(self::$messageSuccess, null, url_to('dash.to', 'certificados/editar/' . $id));
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }
        }
        return $this->render();
    }

    public function editar()
    {
        $id = $this->request->getUri()->getSegment(4, 0);

        $model = new CertificadosModel();

        $entity = $model->where([
            'id' => $id,
            'clientes_id' => Auth::get('id')
        ])->first();

        if (!$entity) {
            return redirect()->route('dash.to', ['certificados?error=' . urlencode(self::$messageDenyAccess)]);
        }

        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'texto_introdutorio' => 'required|max_length[255]',
                'nome' => 'required|max_length[255]',
                'descricao' => 'required',
                'local_emissao' => 'required',
                'data_emissao' => 'required',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            $data = $this->validator->getValidated();
            $data['id'] = $id;
            try {
                $model->save($data);

                $resposta->success(self::$messageSuccess, null, url_to('dash.to', 'certificados/editar/' . $id));
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }
        }

        return $this->render([
            'entity' => $entity
        ]);
    }

    public function deletar()
    {
        $id = $this->request->getUri()->getSegment(4, 0);

        $model = new CertificadosModel();

        $entity = $model->where([
            'id' => $id,
            'clientes_id' => Auth::get('id')
        ])->first();

        $resposta = ApiResponse::init();

        if ($entity) {
            $delete = $model->where([
                'id' => $id,
                'clientes_id' => Auth::get('id')
            ])->delete();

            if ($delete) {
                $resposta->success(self::$messageDeleteSuccess);
            }
        }
        $resposta->error(self::$messageDenyDelete);
    }

    public function modelos()
    {
        $certificadosId = $this->request->getUri()->getSegment(4, 0);
        $id = $this->request->getUri()->getSegment(5, 0);

        $model = new CertificadosModelosModel();

        $entity = $model->where([
            'id' => $id,
            'certificados_id' => $certificadosId,
            'clientes_id' => Auth::get('id')
        ])->first();

        if (!$entity) {
            return redirect()->route('dash.to', ["certificados/editar/{$certificadosId}?error=" . urlencode(self::$messageDenyAccess)]);
        }

        if ($this->request->is('post')) {
            $mode = $this->request->getPost('mode');
            $content = $this->request->getPost('configuracoes');
            if ($this->request->getPost('save')) {
                $resposta = ApiResponse::init();
                try {
                    $entity->mode = $mode;
                    $entity->configuracoes = $content;
                    $model->save($entity);
                    $resposta->success(self::$messageSuccess, $entity);
                } catch (ReflectionException $e) {
                    $resposta->error(self::$messageError, [$e->getMessage()]);
                }
            } else {
                return $this->pdf($entity, $mode, $content);
            }
        }

        return $this->render([
            'entity' => $entity
        ]);
    }

    public function pdf($entity, $mode = null, $content = null, $name = "document.pdf")
    {
        $mode = $mode ?: $entity->mode;
        $mpdf = new \Mpdf\Mpdf(
            [
                'mode' => 'utf-8',
                'orientation' => $mode == 'portrait' ? 'P' : 'L'
            ]
        );
        $frente = $this->render(['entity' => $entity, 'mode' => $mode, 'content' => $content]);
        $mpdf->WriteHTML($frente);
        if (isset($entity->verso)) {
            $verso = '<html><head><style>body{background-size: cover;background-position: center center;}</style></head><body style="background-image: url(' . Arquivo::pastaUpload($entity->verso) . ')"></body></html>';
            $mpdf->AddPage();
            $mpdf->WriteHTML($verso);
        }
        $data = $mpdf->Output($name, \Mpdf\Output\Destination::STRING_RETURN);
        return $this->response->setContentType('application/pdf')
            ->setBody($data);
    }

    public function Dompdf($mode, $content, $name = "document.pdf")
    {
        $options = new \Dompdf\Options();
        $options->set('isHtml5ParserEnabled', true);
        $options->set('isRemoteEnabled', true);
        $dompdf = new \Dompdf\Dompdf($options);
        $html = $this->render(['content' => $content, 'mode' => $mode]);
        $dompdf->loadHtml($html);
        $dompdf->setPaper('A4', $mode);
        $dompdf->render();
        $dompdf->stream($name, ["Attachment" => false]);
//        $data = $dompdf->output();
//        return $this->response->download($name, $data)->inline();
    }

    public function upload()
    {
        $id = $this->request->getPost('modelos_id');
        $model = new CertificadosModelosModel();
        $entity = $model->where([
            'id' => $id,
            'clientes_id' => Auth::get('id')
        ])->first();

        $resposta = ApiResponse::init();

        if (!$entity) {
            $resposta->error(self::$messageDenyAccess);
        }

        foreach ($this->request->getFiles() as $key => $img) {
            if ($img->isValid() && !$img->hasMoved()) {
                $folder = Auth::pastaCliente("certificados/{$entity->certificados_id}/modelos/{$id}/{$key}");
                $name = $img->getRandomName();
                $img->store($folder, $name);
                $entity->$key = $folder . $name;
            }
        }

        try {
            $model->save($entity);
            $resposta->success(self::$messageSuccess, $entity);
        } catch (ReflectionException $e) {
            $resposta->error(self::$messageError, [$e->getMessage()]);
        }
    }

    public function remove()
    {
        $id = $this->request->getUri()->getSegment(5);

        $model = new CertificadosModelosModel();
        $entity = $model->where([
            'id' => $id,
            'clientes_id' => Auth::get('id')
        ])->first();

        $resposta = ApiResponse::init();

        if (!$entity) {
            $resposta->error(self::$messageDenyAccess);
        }

        switch ($this->request->getUri()->getSegment(4)) {
            case "imagem":
                $key = $this->request->getUri()->getSegment(6);
                $entity->$key = null;
                break;
            default:
                $resposta->error(self::$messageDenyDelete);
                break;
        }
        try {
            $model->save($entity);
            $resposta->success(self::$messageSuccess, $entity, previous_url());
        } catch (ReflectionException $e) {
            $resposta->error(self::$messageError, [$e->getMessage()]);
        }
    }

    public function inscritos()
    {
        $id = $this->request->getUri()->getSegment(5);

        $model = new CertificadosModel();
        $entity = $model->where([
            'id' => $id,
            'clientes_id' => Auth::get('id')
        ])->first();

        $resposta = ApiResponse::init();

        if (!$entity) {
            if ($this->request->isAJAX()) {
                $resposta->error(self::$messageDenyAccess);
            } else {
                return redirect()->route('dash.to', ['certificados/editar/' . $id . '?error=' . urlencode(self::$messageDenyAccess)]);
            }
        }

        switch ($this->request->getUri()->getSegment(4)) {
            case "adicionar":
                return $this->adicionarInscrito($entity);
            case "editar":
                return $this->editarInscrito($entity);
            case "importar":
                return $this->importarInscritos($entity);
            case "vincular":
                $this->vincularInscritos($entity);
                break;
            case "progresso":
                $this->progressoInscritos($entity);
                break;
            case "datatable":
                $this->inscritosDatatable($entity);
                break;
            default:
                return redirect()->route('dash.to', ['certificados/editar/' . $entity->id . '?error=' . urlencode(self::$messageDenyAccess)]);
        }
    }

    private function adicionarInscrito(object $entity)
    {
        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'nome' => 'required|max_length[255]',
                'email' => 'required|valid_email|max_length[255]',
                'senha' => 'required|max_length[255]',
                'texto' => 'required',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            $data = $this->validator->getValidated();

            $data['clientes_id'] = $entity->clientes_id;
            $data['certificados_id'] = $entity->id;

            try {
                $model = new CertificadosInscritosModel();
                $model->save($data);

                $resposta->success(self::$messageSuccess, null, url_to('dash.to', 'certificados/inscritos/editar/' . $entity->id . '/' . $model->getInsertID()));
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }

            $resposta->error(self::$messageError);
        }

        return $this->render([
            'entity' => $entity
        ]);
    }

    private function editarInscrito(object $entity)
    {
        $id = $this->request->getUri()->getSegment(6);
        $model = new CertificadosInscritosModel();
        $certificadosInscritos = $model->find($id);

        if (!$certificadosInscritos) {
            return redirect()->route('dash.to', ['certificados/editar/' . $entity->id . '?error=' . urlencode(self::$messageDenyAccess)]);
        }

        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'nome' => 'required|max_length[255]',
                'email' => 'required|valid_email|max_length[255]',
                'senha' => 'required|max_length[255]',
                'texto' => 'required',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            foreach ($this->validator->getValidated() as $key => $value) {
                $certificadosInscritos->$key = $value;
            }

            try {
                $model->save($certificadosInscritos);
                $resposta->success(self::$messageSuccess, null, url_to('dash.to', 'certificados/inscritos/editar/' . $entity->id . '/' . $certificadosInscritos->id));
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }

            $resposta->error(self::$messageError);
        }
        return $this->render([
            'entity' => $entity,
            'certificadosInscritos' => $certificadosInscritos
        ]);
    }

    public function importarInscritos(object $entity)
    {
        $model = new CertificadosArquivosImportadosModel();

        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'arquivo' => 'uploaded[arquivo]|ext_in[arquivo,csv]'
            ];

            if (!$this->validate($rules, [
                'arquivo' => [
                    'ext_in' => t('Somente são aceitos arquivos CSV')
                ]
            ])) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            $data = [];

            foreach ($this->request->getFiles() as $key => $img) {
                if ($img->isValid() && !$img->hasMoved()) {
                    $folder = Auth::pastaCliente("certificados/{$entity->id}/inscritos/{$key}");
                    $name = $img->getRandomName();
                    $img->store($folder, $name);
                    $data[$key] = $folder . $name;
                }
            }

            $data['clientes_id'] = $entity->clientes_id;
            $data['certificados_id'] = $entity->id;

            try {
                $model->save($data);
                $resposta->success(self::$messageSuccess, null, previous_url());
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }

            $resposta->error(self::$messageError);
        }

        $status = [
            StatusEnum::AGUARDANDO->value,
            StatusEnum::PROCESSANDO->value,
            StatusEnum::VINCULADO->value
        ];

        $arquivo = $model->whereIn('status', $status)->first();

        if ($arquivo) {
            $db = \Config\Database::connect();
            $query = $db->query("SHOW COLUMNS FROM certificados_inscritos");
            $colunas = array_filter($query->getResultArray(), function ($array) {
                return !in_array($array['Type'], ["int", "int(11)", "datetime"]);
            });

            try {
                $csv = Reader::createFromPath(Arquivo::pastaUpload($arquivo->arquivo), 'r');
                $csv->setHeaderOffset(0);
//                $csv->setDelimiter(';');
                $columns = $csv->first();
            } catch (InvalidArgument|ReflectionException $e) {
                $arquivo->status = StatusEnum::ERRO->value;
                $model->save($arquivo);
                return redirect()->route('dash.to', ['certificados/inscritos/importar/' . $entity->id . '?error=' . urlencode($e->getMessage())]);
            }
        }

        return $this->render([
            'entity' => $entity,
            'arquivo' => $arquivo,
            'colunas' => $colunas ?? [],
            'columns' => $columns ?? []
        ]);
    }

    public function vincularInscritos(object $entity)
    {
        $model = new CertificadosArquivosImportadosModel();
        $id = $this->request->getPost('id') ?? 0;
        $arquivo = $model->where('vinculo IS NULL')->orWhere('id', $id)->first();

        $resposta = ApiResponse::init();

        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'vinculo' => 'required'
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            $data = $this->validator->getValidated();
            $arquivo->vinculo = Suporte::arrayToJson($data['vinculo']);
            $arquivo->status = StatusEnum::VINCULADO->value;

            try {
                $model->save($arquivo);
                $resposta->success(self::$messageSuccess, $arquivo, previous_url());
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }

            $resposta->error(self::$messageError);
        }
        $resposta->error(self::$messageError, [$arquivo]);
    }

    public function progressoInscritos(object $entity)
    {
        $status = [
            StatusEnum::AGUARDANDO->value,
            StatusEnum::PROCESSANDO->value,
            StatusEnum::VINCULADO->value
        ];

        $model = new CertificadosArquivosImportadosModel();
        $arquivo = $model->whereIn('status', $status)->first();
        $resposta = ApiResponse::init();

        if ($arquivo) {
            try {
                $csv = Reader::createFromPath(Arquivo::pastaUpload($arquivo->arquivo), 'r');
                $csv->setHeaderOffset(0);
//                $csv->setDelimiter(';');

                if (empty($arquivo->total) || $arquivo->total == 0) {
                    $arquivo->porcentagem = 0;
                    $arquivo->quantidade = 0;
                    $arquivo->total = $csv->count();
                }

                $limite = 10;
                $pages = ceil($csv->count() / $limite);
                $page = $this->request->getGet('page') ?? 0;

                if ($page <= $pages) {
                    $stmt = Statement::create()->offset($page)->limit($limite);
                    $result = $stmt->process($csv);


                    $cols = Suporte::jsonToArray($arquivo->vinculo);
                    $modelCertificadoInscrito = new CertificadosInscritosModel();

                    foreach ($result as $row) {
                        $data = [
                            'clientes_id' => Auth::get('id'),
                            'certificados_id' => $entity->id,
                        ];

                        foreach ($cols as $key => $col) {
                            if (isset($row[$col])) {
                                $data[$key] = $row[$col];
                            }
                        }

                        if (isset($data['email'])) {
                            $certificadosInscritos = $modelCertificadoInscrito
                                ->where([
                                    'email' => $data['email'],
                                    'clientes_id' => $data['clientes_id'],
                                    'certificados_id' => $data['certificados_id']
                                ])
                                ->first();

                            if ($certificadosInscritos) {
                                $data['id'] = $certificadosInscritos->id;
                            }
                        }

                        if ($modelCertificadoInscrito->save($data)) {
                            $arquivo->quantidade++;
                        }
                    }

                    $arquivo->porcentagem = Suporte::calculaPorcentagem($arquivo->quantidade, $arquivo->total);

                    $arquivo->status = StatusEnum::PROCESSANDO->value;
                    $model->save($arquivo);

                    $resposta->success(t('Importando'), [
                        'page' => ($page + 1),
                        'pages' => $pages,
                        'percentage' => ceil($arquivo->porcentagem),
                        'finished' => false
                    ]);
                } else {
                    $arquivo->status = StatusEnum::PROCESSADO->value;
                    $model->save($arquivo);
                    $resposta->success(t('Finalizado'), ['finished' => true], url_to('dash.to', 'certificados/editar/' . $entity->id));
                }
            } catch (InvalidArgument|ReflectionException|Exception $e) {
                $arquivo->status = StatusEnum::ERRO->value;
                $model->save($arquivo);
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }
        }

        $resposta->error(self::$messageError);
    }

    public function inscritosDatatable(object $entity)
    {
        $db = Database::connect();

        $table = 'certificados_inscritos';

        $builder = $db->table($table);

        $start = $this->request->getVar('start');
        $length = $this->request->getVar('length');
        $columns = $this->request->getVar('columns');
        $search = $this->request->getVar('search');
        $order = $this->request->getVar('order');
        $clientes_id = $this->request->getVar('clientes_id');

        $dataColumns = [];
        $searchable = null;
        $orderable = null;

        foreach ($columns as $index => $column) {
            if (!$column['data']) {
                continue;
            }

            $dataColumns[] = "{$column['data']} AS " . ($column['name'] ?? $column['data']);

            if ($column['searchable'] && !empty($search['value'])) {
                $searchable[$column['name']] = $search['value'];
            }
            if ($column['orderable']) {
                $orderable[$index] = $column['data'];
            }
        }

        $builder->select($dataColumns);

        if (!empty($searchable)) {
            $builder->orLike($searchable);
        }

        foreach ($order ?? [] as $ord) {
            if (isset($orderable[$ord['column']])) {
                $builder->orderBy($orderable[$ord['column']], $ord['dir']);
            }
        }

        $where = 'deleted_at IS NULL';

        if ($clientes_id) {
            $builder->where('clientes_id', $clientes_id);
            $builder->where('certificados_id', $entity->id);
            $where = "(clientes_id = {$clientes_id} AND certificados_id = {$entity->id} AND deleted_at IS NULL)";
        }

        $data = $builder->where($where)->get($length, $start);

        $recordsTotal = $builder->where($where)->countAllResults();
        $recordsFiltered = $builder->where($where)->countAllResults();

        $datatable = [
            'draw' => (int)$this->request->getVar('draw'),
            'recordsTotal' => (int)$recordsTotal,
            'recordsFiltered' => (int)$recordsFiltered,
            'data' => array_map(function ($item) {
                return array_merge(['DT_RowId' => $item['id']], $item);
            }, $data->getResult('array')),
        ];

        Suporte::json($datatable);
    }

    public function gerar()
    {
        $certificadoId = $this->request->getUri()->getSegment(4);
        $inscritoId = $this->request->getUri()->getSegment(5);
        $clientesId = Auth::get('id');

        try {
            $gerador = GerarCertificado::iniciar($clientesId, $certificadoId, $inscritoId);
            $data = $gerador->gerar('S');
            return $this->response->setContentType('application/pdf')->setBody($data);
        } catch (ExcecaoCertificaSe $e) {
            return redirect()->route('dash.to', ['certificados?error=' . urlencode($e->getMessage())]);
        }
    }

    public function download()
    {
        $certificadoId = $this->request->getUri()->getSegment(4);
        $inscritoId = $this->request->getUri()->getSegment(5);
        $clientesId = Auth::get('id');

        try {
            $gerador = GerarCertificado::iniciar($clientesId, $certificadoId, $inscritoId);
            $gerador->gerar('D');
        } catch (ExcecaoCertificaSe $e) {
            return redirect()->route('dash.to', ['certificados?error=' . urlencode($e->getMessage())]);
        }
    }
}