<?php

namespace App\Controllers\Dashboard;

use App\Controllers\BaseController;
use App\Libraries\ApiResponse;
use App\Libraries\Auth;
use App\Libraries\Suporte;
use App\Models\ClientesPerfilModel;
use ReflectionException;

class MinhaConta extends BaseController
{
    public function index(): string
    {
        return $this->render();
    }

    public function perfil(): string
    {
        $perfil = Auth::perfil();

        if ($this->request->isAJAX()) {
            $resposta = ApiResponse::init();

            $rules = [
                'documento' => 'required|max_length[255]',
                'documento_numero' => 'required|max_length[255]',
                'contatos' => 'required',
                'enderecos' => 'required',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            foreach ($this->validator->getValidated() as $key => $value) {
                $perfil->$key = is_array($value) ? Suporte::arrayToJson($value) : $value;
            }

            try {
                $model = new ClientesPerfilModel();
                $model->save($perfil);
                $resposta->success(self::$messageSuccess, null, previous_url());
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }
        }

        return $this->render([
            'perfil' => $perfil
        ]);
    }
}