<?php

namespace App\Libraries;

use App\Enums\JwtEnum;
use App\Models\ClientesCreditoModel;
use App\Models\ClientesModel;
use App\Models\ClientesPerfilModel;
use App\Models\JwtModel;
use ReflectionException;
use stdClass;

abstract class Auth
{
    protected static array $user = [];

    /**
     * Popula os dados do usuário.
     *
     * @param array $data
     * @return void
     */
    public static function setUser(array $data): void
    {
        foreach ($data as $key => $value) {
            session()->set($key, $value);
        }
    }

    /**
     * Obtém um valor específico do usuário.
     *
     * @param string $key
     * @return mixed|null
     */
    public static function get(string $key): mixed
    {
        return session()->get($key) ?? null;
    }

    public static function pastaCliente(string $subpasta = null): string
    {
        if (is_null($subpasta)) {
            $subpasta = [""];
        } else {
            $subpasta = explode("/", $subpasta);
            $subpasta[] = "";
        }
        return implode("/", array_merge(['clientes', session()->get('id')], $subpasta));
    }

    private static function modelClientes(): ClientesModel
    {
        return new ClientesModel();
    }

    private static function modelPerfil(): ClientesPerfilModel
    {
        return new ClientesPerfilModel();
    }

    private static function modelCreditos(): ClientesCreditoModel
    {
        return new ClientesCreditoModel();
    }

    public static function perfil(): stdClass
    {
        $model = self::modelPerfil();

        $where = [
            'clientes_id' => self::get('id')
        ];

        try {
            $perfil = $model->where($where)->first();
            if (!$perfil) {
                $model->save($where);
                $perfil = new stdClass();
                $perfil->id = $model->getInsertID();
                $perfil->clientes_id = $where['clientes_id'];
            }

            if (!empty($perfil->contatos)) {
                $perfil->contatos = Suporte::jsonToArray($perfil->contatos);
            }

            if (!empty($perfil->enderecos)) {
                $perfil->enderecos = Suporte::jsonToArray($perfil->enderecos);
            }

            return $perfil;
        } catch (ReflectionException $e) {
            return (object)$where;
        }
    }

    public static function user(): object|null
    {
        $model = self::modelClientes();
        $user = $model->find(self::get('id'));
        if (!$user) {
            $user->perfil = self::perfil();
        }
        return $user;
    }

    public static function creditos(): int
    {
        $creditos = 0;
        $model = self::modelCreditos();
        $userCreditos = $model->where('clientes_id', self::get('id'))->first();
        if ($userCreditos) {
            $creditos = $userCreditos->saldo;
        }
        return $creditos;
    }

    /**
     * @throws ReflectionException
     * @throws ExcecaoCertificaSe
     */
    public static function creditosSubtrair(int $quantidade): int
    {
        return self::modelCreditos()->subtrair(self::get('id'), $quantidade);
    }

    public static function revogarAcesso($tipo): bool
    {
        try {
            $jwtModel = new JwtModel();
            return $jwtModel->where([
                'origem_tipo' => JwtEnum::origemPorTipo($tipo),
                'origem_id' => self::get('id'),
            ])->set(['revogado' => JwtEnum::REVOGAR->value])->update();
        } catch (ReflectionException $e) {
            dd($e->getMessage());
        }
    }
}