<?php

namespace App\Libraries;

use ReflectionException;

class Creditos
{
    private int $creditos = 0;

    public function __construct()
    {
        helper('nortedevbr');
        $this->creditos = Auth::creditos();
    }

    public static function init(): static
    {
        return new static();
    }

    /**
     * @throws ExcecaoCertificaSe
     * @throws ReflectionException
     */
    public static function subtrair(int $quantidade = 1): void
    {
        if ($quantidade == 1) {
            self::init()->usarUmCredito();
        } else {
            self::init()->usarCreditos($quantidade);
        }
    }

    /**
     * @throws ReflectionException
     * @throws ExcecaoCertificaSe
     */
    public function usarCreditos(int $quantidade): void
    {
        if ($this->getCreditos() >= $quantidade) {
            $this->setCreditos($quantidade);
        } else {
            $erro = '<strong>ATENÇÃO!</strong><br/>' .
                'Créditos insuficientes para gerar <strong>' . $quantidade . '</strong> certificados.<br/>' .
                'Por favor, adquira novos créditos para continuar.<br/>' .
                '<hr/><a href="' . self::urlComprar($quantidade) . '" class="alert-link">' . t('Adicionar créditos') . '</a>';
            ExcecaoCertificaSe::lancar($erro);
        }
    }

    /**
     * @throws ExcecaoCertificaSe
     * @throws ReflectionException
     */
    public function usarUmCredito(): void
    {
        if ($this->getCreditos() > 0) {
            $this->setCreditos(1);
        } else {
            $erro = '<strong>ATENÇÃO!</strong><br/>' .
                'Seus créditos para gerar certificados se esgotaram.<br/>' .
                'Por favor, adquira novos créditos para continuar.<br/>' .
                '<hr/><a href="' . self::urlComprar() . '" class="alert-link">' . t('Adicionar créditos') . '</a>';
            ExcecaoCertificaSe::lancar($erro);
        }
    }

    public static function quantidadeDisponivel(): int
    {
        return self::init()->getCreditos();
    }

    public static function urlComprar(?int $quantidade = null): string
    {
        return url_to('dash.to', 'creditos/comprar' . ($quantidade ? '?quantidade=' . $quantidade : null));
    }

    public static function urlExtrato(): string
    {
        return url_to('dash.to', 'creditos/extrato');
    }

    public function getCreditos(): int
    {
        return $this->creditos;
    }

    /**
     * @throws ReflectionException
     * @throws ExcecaoCertificaSe
     */
    public function setCreditos(int $usarCreditos): Creditos
    {
        $this->creditos = Auth::creditosSubtrair($usarCreditos);
        return $this;
    }
}