<?php

namespace App\Libraries;

use DateInterval;
use DateTimeImmutable;
use DateTimeZone;
use Exception;
use IntlDateFormatter;

class Data
{
    const FORMAT_DB_DATETIME = "Y-m-d H:i:s";
    const FORMAT_DB_DATE = "Y-m-d";
    const FORMAT_DB_TIME = "H:i:s";
    const FORMAT_BR_MY = "m-Y";

    /**
     * @throws Exception
     */
    public function timezone(): DateTimeZone
    {
        return new DateTimeZone(app_timezone());
    }

    /**
     * @throws Exception
     */
    public function dateObj(string|int $datetime = "now"): DateTimeImmutable
    {
        return new DateTimeImmutable($datetime, $this->timezone());
    }

    public static function agora(string $format = "Y-m-d H:i:s", bool $timestamp = false): int|string
    {
        $obj = new static();
        try {
            $dataAtual = $obj->dateObj("now");
            if ($timestamp) {
                return $dataAtual->getTimestamp();
            }
            return $dataAtual->format($obj->formatByLang($format));
        } catch (Exception $e) {
            return $timestamp ? time() : date($obj->formatByLang($format));
        }
    }

    public static function modificarAgoraPara(string $intervalo, string $format = "Y-m-d H:i:s", bool $timestamp = false): int|string
    {
        $obj = new static();
        try {
            $dataAtual = $obj->dateObj("now");
            if ($timestamp) {
                return $dataAtual->modify($intervalo)->getTimestamp();
            }
            return $dataAtual->modify($intervalo)->format($obj->formatByLang($format));
        } catch (Exception $e) {
            return $timestamp ? strtotime($intervalo) : date($obj->formatByLang($format), strtotime($intervalo));
        }
    }

    public static function porExtenso(string|int $datetime = "now"): string
    {
        $obj = new static();
        try {
            $date = $obj->dateObj($datetime);
            $formatter = new IntlDateFormatter('pt_BR', IntlDateFormatter::LONG, IntlDateFormatter::NONE);
            return $formatter->format($date);
        } catch (Exception $e) {
            return strtotime($datetime);
        }
    }

    private function formatByLang(string $format): string
    {
        return $format;
    }

    public static function obj(int|string $datetime = "now"): ?DateTimeImmutable
    {
        try {
            return (new static())->dateObj($datetime);
        } catch (Exception $e) {
            return null;
        }
    }
}