<?php

namespace App\Models;

use CodeIgniter\Model;
use ReflectionException;
use stdClass;

/**
 * @property int $id
 * @property string $nome
 * @property string $email
 * @property string $senha
 * @property string $created_at
 * @property string $updated_at
 */
class ClientesModel extends Model
{
    protected $table = 'clientes';
    protected $primaryKey = 'id';
    protected $useAutoIncrement = true;
    protected $returnType = 'object';
    protected $useSoftDeletes = true;
    protected $protectFields = true;
    protected $allowedFields = [
        'id',
        'nome',
        'email',
        'senha',
        'created_at',
        'updated_at',
    ];

    protected bool $allowEmptyInserts = false;
    protected bool $updateOnlyChanged = true;

    protected array $casts = [];
    protected array $castHandlers = [];

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat = 'datetime';
    protected $createdField = 'created_at';
    protected $updatedField = 'updated_at';
    protected $deletedField = 'deleted_at';

    // Validation
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert = [];
    protected $afterInsert = [];
    protected $beforeUpdate = [];
    protected $afterUpdate = [];
    protected $beforeFind = [];
    protected $afterFind = [];
    protected $beforeDelete = [];
    protected $afterDelete = [];

    public function verificarSenha(stdClass $cliente, string|array $senha): bool
    {
        if (is_array($senha)) {
            if (key_exists('senha', $senha)) {
                $senha = $senha['senha'];
            } else {
                return false;
            }
        }

        if (password_verify($senha, $cliente->senha)) {
            if (password_needs_rehash($cliente->senha, PASSWORD_DEFAULT)) {
                try {
                    $this->save([
                        'id' => $cliente->id,
                        'senha' => password_hash($senha, PASSWORD_DEFAULT)
                    ]);
                } catch (ReflectionException $e) {
                    return false;
                }
            }
            return true;
        }

        return false;
    }
}
