<?php

namespace App\Models;

use CodeIgniter\Model;
use ReflectionException;
use stdClass;

class InscritosModel extends Model
{
    protected $table = 'inscritos';
    protected $primaryKey = 'id';
    protected $useAutoIncrement = true;
    protected $returnType = 'object';
    protected $useSoftDeletes = true;
    protected $protectFields = true;
    protected $allowedFields = [
        'id',
        'nome',
        'email',
        'senha',
        'senha_hash',
        'created_at',
        'updated_at',
    ];

    protected bool $allowEmptyInserts = false;
    protected bool $updateOnlyChanged = true;

    protected array $casts = [];
    protected array $castHandlers = [];

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat = 'datetime';
    protected $createdField = 'created_at';
    protected $updatedField = 'updated_at';
    protected $deletedField = 'deleted_at';

    // Validation
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert = [];
    protected $afterInsert = [];
    protected $beforeUpdate = ['beforeUpdate'];
    protected $afterUpdate = [];
    protected $beforeFind = [];
    protected $afterFind = ['getCertificados'];
    protected $beforeDelete = [];
    protected $afterDelete = [];

    public function beforeUpdate($data)
    {
        if (isset($data['data']->senha) && !empty(isset($data['data']->senha))) {
            $data['data']->senha_hash = password_hash($data['data']->senha, PASSWORD_DEFAULT);
        }
        return $data;
    }

    public function getCertificados($data)
    {
        if (isset($data['data']->certificados_id) && $data['method'] === "first") {
            $model = new CertificadosModel();
            $entity = $model->find($data['data']->certificados_id);
            $data['data']->Certificados = $entity;
        }
        return $data;
    }

    public function verificarSenha(stdClass $inscrito, string|array $senha): bool
    {
        if (is_array($senha)) {
            if (key_exists('senha', $senha)) {
                $senha = $senha['senha'];
            } else {
                return false;
            }
        }

        if (password_verify($senha, $inscrito->senha_hash)) {
            if (password_needs_rehash($inscrito->senha_hash, PASSWORD_DEFAULT)) {
                try {
                    $this->save([
                        'id' => $inscrito->id,
                        'senha_hash' => password_hash($senha, PASSWORD_DEFAULT)
                    ]);
                } catch (ReflectionException $e) {
                    return false;
                }
            }
            return true;
        }

        return false;
    }
}
