<?php

namespace App\ThirdParty\Cobrancas;

use App\Libraries\ExcecaoCertificaSe;
use App\Models\CobrancasModel;
use App\ThirdParty\Cobrancas\DTO\CobrancaDTO;
use App\ThirdParty\Cobrancas\Integradores\IntegradorInterface;

class Cobranca
{
    const FAKE = 'FAKE';
    const SAFE2PAY = 'Safe2Pay';

    const INTEGRADORES = [
        self::FAKE => \App\ThirdParty\Cobrancas\Integradores\FAKE::class,
        self::SAFE2PAY => \App\ThirdParty\Cobrancas\Integradores\Safe2Pay::class,
    ];

    private string $integrador;
    private IntegradorInterface $instance;
    private ?CobrancaDTO $cobranca = null;

    /**
     * @throws ExcecaoCertificaSe
     */
    public function __construct(string $integrador, ?CobrancaDTO $cobranca = null)
    {
        $this->setIntegrador($integrador);
        $this->setCobranca($cobranca);
        $this->instantiate();
    }

    public function getCobranca(): ?CobrancaDTO
    {
        return $this->cobranca;
    }

    public function setCobranca(?CobrancaDTO $cobranca): Cobranca
    {
        $this->cobranca = $cobranca;
        return $this;
    }

    /**
     * @throws ExcecaoCertificaSe
     */
    private function instantiate(): void
    {
        if (key_exists($this->getIntegrador(), self::INTEGRADORES)) {
            $class = self::INTEGRADORES[$this->getIntegrador()];
            $this->instance = new $class($this->getCobranca());

            $model = new CobrancasModel();
        } else {
            ExcecaoCertificaSe::lancar('Integrador não identificado');
        }
    }

    /**
     * @throws ExcecaoCertificaSe
     */
    public static function init(string $integrador, ?CobrancaDTO $cobranca = null): IntegradorInterface
    {
        return (new static($integrador, $cobranca))->instance();
    }

    public function getIntegrador(): string
    {
        return $this->integrador;
    }

    public function setIntegrador(string $integrador): Cobranca
    {
        $this->integrador = $integrador;
        return $this;
    }

    public function instance(): IntegradorInterface
    {
        return $this->instance;
    }

}