<?php

namespace App\ThirdParty\Cobrancas\Integradores;

use App\Libraries\Auth;
use App\Libraries\Data;
use App\Libraries\Suporte;
use App\ThirdParty\Cobrancas\DTO\CobrancaDTO;
use App\ThirdParty\Cobrancas\DTO\ConsultaPagamentoDTO;
use App\ThirdParty\Cobrancas\DTO\WebhookDTO;

class Safe2Pay extends IntegradorAbstract implements IntegradorInterface
{

    /**
     * @param CobrancaDTO $cobranca
     * @return $this
     */
    public function criarCobrancaCartaoCredito(CobrancaDTO $cobranca): static
    {
        return $this;
    }

    /**
     * @param CobrancaDTO $cobranca
     * @return $this
     */
    public function criarCobrancaBoleto(CobrancaDTO $cobranca): static
    {
        $products = [];

        foreach ($cobranca->getProdutos() as $produto) {
            $products[] = [
                "Code" => $produto->id,
                "Description" => $produto->nome,
                "UnitPrice" => Suporte::intToFloat($produto->valor),
                "Quantity" => 1
            ];
        }

        $payload = [
            "IsSandbox" => $cobranca->isHomologacao(),
            "Application" => "FAKE_APLICACAO",
            "Vendor" => "Certifica-se",
            "CallbackUrl" => $cobranca->getUrlCallback(),
            "PaymentMethod" => "1",// Boleto Bancário
            "Reference" => "TESTE",
            "Customer" => [
                "Name" => Auth::get('nome'),
                "Identity" => Auth::perfil()->documento_numero,
                "Phone" => Auth::perfil()->contatos->celular->numero ?? Auth::perfil()->contatos->residencial,
                "Email" => Auth::get('email'),
                "Address" => [
                    "ZipCode" => Auth::perfil()->enderecos->cobranca->cep ?? Auth::perfil()->enderecos->residencial->cep,
                    "Street" => Auth::perfil()->enderecos->cobranca->logradouro ?? Auth::perfil()->enderecos->residencial->logradouro,
                    "Number" => Auth::perfil()->enderecos->cobranca->numero ?? Auth::perfil()->enderecos->residencial->numero,
                    "Complement" => Auth::perfil()->enderecos->cobranca->complemento ?? Auth::perfil()->enderecos->residencial->complemento,
                    "District" => Auth::perfil()->enderecos->cobranca->bairro ?? Auth::perfil()->enderecos->residencial->bairro,
                    "CityName" => Auth::perfil()->enderecos->cobranca->localidade ?? Auth::perfil()->enderecos->residencial->localidade,
                    "StateInitials" => Auth::perfil()->enderecos->cobranca->uf ?? Auth::perfil()->enderecos->residencial->uf,
                    "CountryName" => Auth::perfil()->enderecos->cobranca->pais ?? Auth::perfil()->enderecos->residencial->pais,
                ]
            ],
            "Products" => $products,
            "PaymentObject" => [
                "DueDate" => Data::obj()->modify('+7 day')->format('Y-m-d'),
                "Instruction" => "Instrução de Exemplo",
                "Message" => [
                    "Mensagem 1",
                    "Mensagem 2",
                    "Mensagem 3"
                ],
                "PenaltyRate" => 2,
                "InterestRate" => 0.4,
                "CancelAfterDue" => false,
                "IsEnablePartialPayment" => false,
                //"DiscountType" => "1",
                //"DiscountAmount" => 5,
                //"DiscountDue" => "30/09/2021"
            ]
        ];


        return $this;
    }

    /**
     * @param CobrancaDTO $cobranca
     * @return $this
     */
    public function criarCobrancaPix(CobrancaDTO $cobranca): static
    {
        return $this;
    }

    /**
     * @param int|string|null $transacaoId
     * @param CobrancaDTO|null $cobranca
     * @return $this
     */
    public function consultarCobranca(int|string|null $transacaoId = null, ?CobrancaDTO $cobranca = null): static
    {
        return $this;
    }

    /**
     * @param int|string|null $transacaoId
     * @param CobrancaDTO|null $cobranca
     * @return $this
     */
    public function cancelarCobranca(int|string|null $transacaoId = null, ?CobrancaDTO $cobranca = null): static
    {
        return $this;
    }

    /**
     * @param ConsultaPagamentoDTO $consultaPagamento
     * @return $this
     */
    public function consultarPagamentos(ConsultaPagamentoDTO $consultaPagamento): static
    {
        return $this;
    }

    /**
     * @param WebhookDTO $webhook
     * @return $this
     */
    public function webhook(WebhookDTO $webhook): static
    {
        return $this;
    }
}