<?= $this->extend('application/_layout') ?>

<?= $this->section('content') ?>

    <div class="container-fluid">

        <div class="row g-3 my-3 justify-content-center" id="listaCertificados">

            <div class="col-12">
                <div class="d-flex justify-content-center">
                    <div class="spinner-border" role="status">
                        <span class="visually-hidden">Loading...</span>
                    </div>
                </div>
            </div>

        </div>

        <div class="row g-3">
            <div class="col-md-12 d-flex justify-content-center align-items-center">
                <nav aria-label="Page navigation example">
                    <ul class="pagination justify-content-center">
                        <li class="page-item">
                            <a class="page-link" href="#" id="previous"><?= t('Anterior') ?></a>
                        </li>
                        <li class="page-item active" aria-current="page">
                            <span class="page-link"><?= tf('Página: %d', request()->getGet('page') ?? 1); ?></span>
                        </li>
                        <li class="page-item">
                            <a class="page-link" href="#" id="next"><?= t('Próximo') ?></a>
                        </li>
                    </ul>
                </nav>

                <!--
                <button type="button" class="btn btn-theme" id="loadMoreBtn">
                    <?= t('Ver mais') ?>
                </button>
                -->
            </div>
        </div>
    </div>

<?= $this->endSection() ?>

<?= $this->section('css') ?>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/animate.css/4.1.1/animate.min.css"/>
<?= $this->endSection() ?>

<?= $this->section('js') ?>
    <script type="module">
        import {AjaxRequest} from '/assets/js/AjaxRequest.js';
        import {loadBtnBeforeClick} from '/assets/js/Global.js';

        moment.locale(LOCALE.toLowerCase()); // Defina o idioma desejado

        document.addEventListener('DOMContentLoaded', () => {
            // Torna a função global para que possa ser chamada a partir do botão
            // window.loadCertificados = loadCertificados;

            // Carrega certificados inicialmente
            loadCertificados(<?= request()->getGet('page') ?? 1;?>);
        });

        const ajaxRequest = new AjaxRequest();
        const url = new URL(window.location.href);

        function htmlStringToDOM(htmlString) {
            const tempDiv = document.createElement('div');
            tempDiv.innerHTML = htmlString;
            return tempDiv.firstElementChild;
        }

        async function navigation(ev) {
            // ev.preventDefault();
            let btnResult = loadBtnBeforeClick(ev.target);
            // const pageUrl = new URL(ev.target.href);
            // const params = new URLSearchParams(pageUrl.search);
            // const queryParams = Object.fromEntries(params.entries());
            // await loadCertificados(queryParams.page);
            btnResult.reset();
        }

        async function loadCertificados(page = 1) {
            try {
                let formData = new FormData();
                formData.append('page', page);
                const response = await ajaxRequest.post(url.pathname, formData);
                console.log(response.data);

                const lista = document.getElementById('listaCertificados');
                const previous = document.getElementById('previous');
                const next = document.getElementById('next');
                lista.innerHTML = '';

                const navigationHandler = ev => navigation(ev);

                // Atualiza o botão "Anterior"
                updateNavigationButton(previous, response.meta.previous, navigationHandler);

                // Atualiza o botão "Próximo"
                updateNavigationButton(next, response.meta.next, navigationHandler);

                // Renderiza os certificados com delay
                await renderCertificadosWithDelay(response.data, 100);
            } catch (error) {
                console.error('Erro ao fazer POST:', error);
            }
        }

        function updateNavigationButton(button, url, handler) {
            button.removeEventListener('click', handler);
            if (url === null) {
                button.classList.add('disabled');
                button.setAttribute('href', '#');
            } else {
                button.classList.remove('disabled');
                button.setAttribute('href', url);
                button.addEventListener('click', handler);
            }
        }

        function renderCertificadosWithDelay(certificados, delay) {
            return certificados.reduce((promise, certificado) => {
                return promise.then(() => {
                    renderCertificado(certificado);
                    return new Promise(resolve => setTimeout(resolve, delay));
                });
            }, Promise.resolve());
        }

        function renderCertificado(certificado) {
            const lista = document.getElementById('listaCertificados');
            const htmlString = `
                <div class="col-md-3 col-xs-4 col-6 animate__animated animate__backInDown animate__delay-1s">
                    <div class="card card-body">
                        <h4>${certificado.Certificado.nome}</h4>
                        <p class="m-0 p-0">Local e data de emissão:<br/>
                            ${certificado.Certificado.local_emissao}<br/>
                            ${moment(certificado.Certificado.data_emissao).format('L')}
                        </p>
                        ${certificado.CertificadoModelo.frente ? `
                        <img class="img-fluid rounded" style="max-width: 450px; width: 100%; height: 250px; object-fit: cover;"
                             src="<?=url_to('arquivo.imagem', '/')?>${certificado.CertificadoModelo.frente}"
                             alt="${certificado.Certificado.nome}">` : ''}
                        <a href="<?=url_to('app.to', 'certificado/')?>${certificado.codigo_validacao}" class="btn btn-theme mt-2" target="_blank">
                            <i class="bi bi-download me-2"></i>Baixar
                        </a>
                    </div>
                </div>`;
            const tag = htmlStringToDOM(htmlString);
            lista.append(tag);
        }

        function renderCertificados(certificados) {
            certificados.forEach(certificado => renderCertificado(certificado));
        }
    </script>


    <!--    <script>-->
    <!--        document.getElementById('loadMoreBtn').addEventListener('click', () => {-->
    <!--            loadCertificados(true);-->
    <!--        });-->
    <!--    </script>-->
<?= $this->endSection() ?>