<?php

use App\Enums\StatusCertificadosEnum;
use App\Libraries\Suporte;

?>
<?= $this->extend('dashboard/_layout') ?>

<?= $this->section('content') ?>

<div class="container-fluid">
    <div class="row g-3">
        <div class="col-12 d-flex justify-content-between align-items-center">
            <h1>
                <?= tf(ucfirst($sys->action) . ' %s', t('certificado')) ?><br/>
                <?php if ($sys->isEdit): ?>
                    <span class="text-white-50 fs-6 m-0 p-0">
                     ref.: <?= $entity->id ?>
                    </span>
                <?php endif; ?>
            </h1>
            <div class="btn-toolbar" role="toolbar" aria-label="<?= t('Barra de ferramentas com grupos de botões') ?>">
                <div class="btn-group btn-group-sm" role="group" aria-label="<?= t('Grupo padrão') ?>">
                    <a href="<?= url_to('dash.to', 'certificados') ?>" class="btn btn-outline-theme">
                        <i class="bi bi-chevron-left me-1"></i>
                        <?= t('Voltar') ?>
                    </a>
                    <?php if ($sys->isEdit): ?>
                        <a href="<?= url_to('dash.to', 'certificados/novo') ?>" class="btn btn-theme">
                            <i class="bi bi-plus me-1"></i>
                            <?= t('Novo') ?>
                        </a>
                    <?php endif; ?>
                </div>
            </div>
        </div>
        <div class="col-12" id="notificacao"></div>
        <div class="<?= $sys->isEdit ? 'col-md-6' : 'col-12' ?>">
            <div class="card card-body">
                <form id="save" method="post" class="row g-3" enctype="multipart/form-data">
                    <div class="<?= $sys->isEdit ? 'col-md-12' : 'col-md-6' ?>">
                        <label for="nome"><?= t('Nome') ?></label>
                        <input type="text" class="form-control" id="nome" name="nome">
                    </div>
                    <div class="<?= $sys->isEdit ? 'col-md-12' : 'col-md-6' ?>">
                        <label for="descricao"><?= t('Descrição') ?></label>
                        <input type="text" class="form-control" id="descricao" name="descricao">
                    </div>
                    <div class="col-md-4">
                        <label for="texto_introdutorio"><?= t('Texto introdutório') ?></label>
                        <input type="text" class="form-control" id="texto_introdutorio" name="texto_introdutorio"
                               value="Certificado">
                    </div>
                    <div class="col-md-4">
                        <label for="local_emissao"><?= t('Local emissão') ?></label>
                        <input type="text" class="form-control" id="local_emissao" name="local_emissao">
                    </div>
                    <div class="col-md-4">
                        <label for="data_emissao"><?= t('Data emissão') ?></label>
                        <input type="date" class="form-control" id="data_emissao" name="data_emissao">
                    </div>
                    <div class="<?= $sys->isEdit ? 'col-md-12' : 'col-md-6' ?>">
                        <button type="submit" class="btn btn-theme">
                            <i class="bi bi-floppy me-1"></i>
                            <?= t($sys->isEdit ? 'Atualizar' : 'Salvar') ?>
                        </button>
                    </div>
                </form>
            </div>
        </div>

        <?php if ($sys->isEdit): ?>

            <div class="<?= $sys->isEdit ? 'col-md-6' : 'col-12' ?>">
                <div class="card card-body">
                    <div class="d-flex justify-content-between align-items-center gap-2">
                        <h5 class="card-title">
                            <?= t('Modelo do certificado') ?>
                        </h5>
                        <div>
                            <a href="<?= url_to('dash.to', "certificados/modelos/{$entity->id}/{$entity->CertificadosModelos->id}") ?>"
                               class="btn btn-theme">
                                <i class="bi bi-pencil-square me-1"></i>
                                <?= t('Editar modelo') ?>
                            </a>
                        </div>
                    </div>
                    <div class="d-flex justify-content-start align-items-start gap-2 flex-column flex-md-row">
                        <?php foreach (['frente', 'verso'] as $tipo):
                            if (isset($entity->CertificadosModelos->$tipo)):
                                ?>
                                <div class="d-flex flex-column align-items-start position-relative">
                                    <p class="lead m-0 p-0">
                                        <?= t(ucfirst($tipo)) ?>
                                    </p>
                                    <?php
                                    $img = null;
                                    if (!empty($entity->CertificadosModelos->$tipo)) {
                                        $img = url_to('arquivo.imagem', $entity->CertificadosModelos->$tipo);
                                    }
                                    $certificadoFrente = [
                                        'src' => $img ?? 'https://placehold.co/300x200?text=' . t(ucfirst($tipo)),
                                        'alt' => 'Certificado ' . $tipo,
                                        'class' => 'img-fluid certificado-' . $tipo,
                                        'width' => '300',
                                        'height' => '200',
                                        'style' => 'width: 300px; height: 200px; object-fit: cover;'
                                    ];
                                    echo img($certificadoFrente);
                                    if ($img):
                                        ?>
                                        <a href="<?= url_to('dash.to', 'certificados/remove/imagem/' . $entity->id . '/' . $tipo) ?>"
                                           class="btn btn-sm btn-danger rounded-circle position-absolute bottom-0 delete"
                                           data-bs-toggle="tooltip"
                                           data-bs-title="<?= tf('Remover imagem de fundo: %s', t(ucfirst($tipo))) ?>">
                                            <i class="bi bi-trash"></i>
                                        </a>
                                    <?php endif; ?>
                                </div>
                            <?php
                            endif;
                        endforeach; ?>
                    </div>
                </div>
            </div>

            <div class="col-12">
                <div class="card card-body">
                    <div class="d-flex justify-content-between align-items-center gap-2 mb-2">
                        <h5 class="card-title">
                            <?= t('Inscritos') ?>
                        </h5>
                        <div class="btn-toolbar">
                            <div id="GerarCertificados" class="btn-group btn-group-sm me-2 d-none">
                                <a href="<?= url_to('dash.to', "certificados/fila/{$entity->id}") ?>"
                                   class="btn btn-primary fila">
                                    <i class="bi bi-gear-wide me-1"></i>
                                    <?= t('Gerar certificados') ?>
                                </a>
                            </div>
                            <div class="btn-group btn-group-sm">
                                <a href="<?= url_to('dash.to', "certificados/inscritos/adicionar/{$entity->id}") ?>"
                                   class="btn btn-theme">
                                    <i class="bi bi-person-add me-1"></i>
                                    <?= t('Adicionar') ?>
                                </a>
                                <a href="<?= url_to('dash.to', "certificados/inscritos/importar/{$entity->id}") ?>"
                                   class="btn btn-success">
                                    <i class="bi bi-people me-1"></i>
                                    <?= t('Importar') ?>
                                </a>
                            </div>
                        </div>
                    </div>

                    <div class="table-responsive">
                        <table id="tbl" class="table table-dark table-hover table-striped" style="width: 100%"></table>
                    </div>
                </div>
            </div>
        <?php endif; ?>

    </div>
</div>

<?= $this->endSection() ?>

<?= $this->section('js') ?>
<script type="module">
    import {AjaxRequest} from '/assets/js/AjaxRequest.js';
    import {DataTableButtons} from '/assets/js/DataTableButtons.js';
    import {formSubmit, populateForm} from '/assets/js/Global.js';

    let formName = 'form[id=save]';
    formSubmit(formName, 'notificacao');
    populateForm(formName, <?=Suporte::arrayToJson($entity ?? null)?>);

    <?php if ($sys->isEdit): ?>
    const ajaxRequest = new AjaxRequest();
    const currentUrl = new URL(window.location.href);

    const table = new DataTable('#tbl', {
        responsive: true,
        processing: true,
        search: {
            return: true
        },
        serverSide: true,
        language: {
            url: `/assets/i18n/${LOCALE}.json`,
        },
        ajax: {
            url: '<?=url_to('dash.to', "certificados/inscritos/datatable/{$entity->id}")?>',
            headers: {
                'X-Requested-With': 'XMLHttpRequest',
                'Authorization': `Bearer ${ajaxRequest.jwt()}`
            },
            type: 'POST',
            data: function (d) {
                d.origem = 'certificados';
                d.clientes_id = ajaxRequest.userData()?.id;
            }
        },
        order: [[0, 'desc']],
        columns: [
            {name: 'id', data: 'id', title: '<?=t('Inscrição')?>'},
            {name: 'inscritos_id', data: 'inscritos_id', title: '<?=t('Usuário')?>'},
            {
                name: 'gerado', data: 'gerado', title: '<?=t('Gerado')?>', render: (data) => {
                    let status = <?=Suporte::arrayToJson(StatusCertificadosEnum::toArray())?>;
                    return status[data] ?? NaN;
                }
            },
            {name: 'nome', data: 'nome', title: '<?=t('Nome')?>'},
            {name: 'email', data: 'email', title: '<?=t('E-mail')?>'},
            {name: 'texto', data: 'texto', title: '<?=t('Texto do certificado')?>'},
            {
                name: null,
                data: null,
                width: '50px',
                orderable: false,
                searchable: false,
                render: (data, type, row, meta) => {
                    let gerado = <?=Suporte::arrayToJson(StatusCertificadosEnum::statusGerado())?>;

                    const btn = DataTableButtons.init(data, type, row, meta);

                    if (gerado.includes(parseInt(row.gerado))) {
                        btn.add('<?=t('Baixar certificado')?>', '<?=url_to('dash.to', "certificados/download/{$entity->id}/")?>' + data.id, 'bi bi-download me-1', 'text-success')
                            .divider();
                    } else {
                        btn.add('<?=t('Gerar certificado')?>', '<?=url_to('dash.to', "certificados/gerar/{$entity->id}/")?>' + data.id, 'bi bi-gear-wide me-1', 'text-success')
                            .divider();
                    }

                    btn
                        .add('<?=t('Editar')?>', '<?=url_to('dash.to', "certificados/inscritos/editar/{$entity->id}/")?>' + data.id, 'bi bi-pencil-square me-1')
                        .divider()
                        .add('<?=t('Deletar')?>', "'javascript:alert('em desenvolvimento');'", 'bi bi-trash me-1', 'delete text-danger');

                    return btn.render();
                }
            },
        ],
        drawCallback: settings => {
            const api = new $.fn.dataTable.Api(settings);
            if (api.rows().data().length > 0) {
                document.getElementById('GerarCertificados').classList.remove('d-none');
            }
        }
    });
    <?php endif;?>
</script>
<?= $this->endSection() ?>
