<?php

use App\Controllers\Dashboard\Certificados;

?>
<?= $this->extend('dashboard/_layout') ?>

<?= $this->section('content') ?>
<?php
$importar = false;
?>
<div class="container-fluid">
    <div class="row g-3">
        <div class="col-12 d-flex justify-content-between align-items-center">
            <h1>
                <?= tf('Importar  %s', t('inscritos')) ?><br/>
                <span class="text-white-50 fs-6 m-0 p-0">
                     ref.: <?= $entity->id . " - " . $entity->nome ?>
                </span>
            </h1>
            <div class="btn-toolbar" role="toolbar" aria-label="<?= t('Barra de ferramentas com grupos de botões') ?>">
                <div class="btn-group btn-group-sm" role="group" aria-label="<?= t('Grupo padrão') ?>">
                    <a href="<?= url_to('dash.to', 'certificados/editar/' . $entity->id) ?>"
                       class="btn btn-outline-theme">
                        <i class="bi bi-chevron-left me-1"></i>
                        <?= t('Voltar') ?>
                    </a>
                </div>
            </div>
        </div>

        <div class="col-12" id="notificacao"></div>

        <?php if (empty($arquivo)): ?>
            <div class="col-12">
                <p class="lead">
                    <?= t('O arquivo deve conter as seguintes colunas: nome, email, senha, texto.') ?><br/>
                    <?= t('As colunas devem ser separadas por ponto e vírgula ( ; )') ?>
                </p>
                <div class="card card-body">
                    <form id="importar-arquivo" class="row g-3" method="post" enctype="multipart/form-data">

                        <div class="col-md-12">
                            <div class="input-group">
                                <label for="arquivo" class="input-group-text"><?= t('Enviar arquivo') ?></label>
                                <input id="arquivo" name="arquivo" type="file" class="form-control" accept=".csv">

                                <button type="submit" class="btn btn-success">
                                    <i class="bi bi-upload me-1"></i>
                                    <?= t('Enviar') ?>
                                </button>
                            </div>
                        </div>

                    </form>
                </div>
            </div>
        <?php elseif (empty($arquivo->vinculo)): ?>

            <div class="col-12">
                <p class="lead">
                    <?= t('Por favor, defina a ordem dos valores') ?>
                </p>

                <div class="card card-body">
                    <form action="<?= url_to('dash.to', "certificados/inscritos/vincular/{$entity->id}") ?>"
                          id="importar-arquivo" class="row g-3" method="post" enctype="multipart/form-data">

                        <?php foreach ($colunas as $coluna):
                            if (!in_array($coluna['Field'], Certificados::NAO_EXIBIR_COLUNA_NO_VINCULO)):
                                ?>
                                <div class="col-md-6">
                                    <div class="input-group">
                                        <label class="input-group-text"
                                               for="<?= $coluna['Field'] ?>"><?= $coluna['Field'] ?></label>
                                        <select class="form-select" id="<?= $coluna['Field'] ?>"
                                                name="vinculo[<?= $coluna['Field'] ?>]">
                                            <option selected><?= t('Selecione...') ?></option>
                                            <?php foreach ($columns as $name => $column): ?>
                                                <option value="<?= $name ?>"><?= $name . " - (" . word_limiter($column, 4) . ")" ?></option>
                                            <?php endforeach; ?>
                                        </select>
                                    </div>
                                </div>
                            <?php endif;
                        endforeach; ?>

                        <div class="col-md-12">
                            <button type="submit" class="btn btn-success">
                                <i class="bi bi-upload me-1"></i>
                                <?= t('Iniciar importação') ?>
                            </button>
                        </div>

                    </form>
                </div>
            </div>
        <?php
        else:
            $importar = true;
            ?>
            <div class="col-12">
                <p class="lead">
                    <?= t('Por favor, aguarde...') ?>
                </p>

                <div class="card card-body">

                    <div class="progress" role="progressbar" aria-label="Animated striped example" aria-valuenow="0"
                         aria-valuemin="0" aria-valuemax="100" style="height: 20px">
                        <div class="progress-bar progress-bar-striped progress-bar-animated bg-success"
                             style="width: 0">
                            0%
                        </div>
                    </div>

                </div>
            </div>
        <?php endif; ?>
    </div>
</div>

<?= $this->endSection() ?>

<?= $this->section('js') ?>
<script type="module">
    import {formSubmit, populateForm} from '/assets/js/Global.js';
    import {AjaxRequest} from '/assets/js/AjaxRequest.js';

    let importar = <?=$importar ? 'true' : 'false'?>;

    if (importar) {
        processar();
    } else {
        let formName = 'form[id=importar-arquivo]';
        formSubmit(formName, 'notificacao');
        //populateForm(formName, <?php //=\App\Libraries\Suporte::arrayToJson($certificadosInscritos ?? null)?>//);
    }

    async function processar(page = null) {
        const ajaxRequest = new AjaxRequest();
        try {
            const url = '<?=url_to('dash.to', "certificados/inscritos/progresso/{$entity->id}")?>';
            const data = await ajaxRequest.get(url + (page ? '?page=' + page : ''));
            if (data.data.percentage) {
                let percentage = data.data.percentage;
                let progress = document.querySelector('.progress');
                let progressBar = document.querySelector('.progress-bar');
                progress.setAttribute('aria-valuenow', percentage);
                progressBar.style.width = `${percentage}%`;
                progressBar.textContent = `${percentage}%`;
            }
            if (data.data.page <= (data.data.pages + 1)) {
                await processar(data.data.page);
            }
            if (data.data.finished) {
                swal.fire({
                    text: data.message,
                    icon: !data.success ? "error" : "success",
                    toast: true,
                    showConfirmButton: false,
                    timer: !data.success ? 5000 : 1500,
                    timerProgressBar: true,
                }).then(() => {
                    swal.showLoading();
                    window.location.href = '<?= url_to('dash.to', 'certificados/editar/' . $entity->id) ?>';
                });
            }
        } catch (error) {
            console.error('Erro ao fazer GET:', error);
        }
    }
</script>
<?= $this->endSection() ?>
