<?= $this->extend('dashboard/_layout') ?>

<?= $this->section('content') ?>

<div class="container-fluid">
    <div class="row g-3">
        <div class="col-12 d-flex justify-content-between align-items-center">
            <h1><?= t('Certificados') ?></h1>
            <div class="btn-toolbar" role="toolbar" aria-label="<?= t('Barra de ferramentas com grupos de botões') ?>">
                <div class="btn-group btn-group-sm" role="group" aria-label="<?= t('Grupo padrão') ?>">
                    <a href="<?= current_url() . "/novo" ?>" class="btn btn-theme">
                        <i class="bi bi-plus me-1"></i>
                        <?= t('Novo') ?>
                    </a>
                </div>
            </div>
        </div>
        <div class="col-12">
            <div class="card card-body">
                <div class="table-responsive">
                    <table id="tbl" class="table table-dark table-hover table-striped" style="width: 100%"></table>
                </div>
            </div>
        </div>
    </div>
</div>

<?= $this->endSection() ?>

<?= $this->section('js') ?>
<script type="module">
    import {AjaxRequest} from '/assets/js/AjaxRequest.js';
    import {DataTableButtons} from '/assets/js/DataTableButtons.js';

    const ajaxRequest = new AjaxRequest();
    const currentUrl = new URL(window.location.href);

    const table = new DataTable('#tbl', {
        responsive: true,
        processing: true,
        search: {
            return: true
        },
        serverSide: true,
        language: {
            url: `/assets/i18n/${LOCALE}.json`,
        },
        ajax: {
            url: currentUrl.pathname + '/datatable',
            headers: {
                'X-Requested-With': 'XMLHttpRequest',
                'Authorization': `Bearer ${ajaxRequest.jwt()}`
            },
            type: 'POST',
            data: function (d) {
                d.origem = 'certificados';
                d.clientes_id = ajaxRequest.userData()?.id;
            }
        },
        order: [[0, 'desc']],
        columns: [
            {name: 'id', data: 'id', title: 'ID'},
            {name: 'nome', data: 'nome', title: '<?=t('Nome')?>'},
            {name: 'descricao', data: 'descricao', title: '<?=t('Descrição')?>'},
            {
                name: null,
                data: null,
                width: '50px',
                orderable: false,
                searchable: false,
                render: (data, type, row, meta) => {
                    return DataTableButtons.init(data, type, row, meta)
                        .add('Editar', currentUrl.pathname + '/editar/' + data.id, 'bi bi-pencil-square me-1')
                        .divider()
                        .add('Deletar', currentUrl.pathname + '/deletar/' + data.id, 'bi bi-trash me-1', 'delete text-danger')
                        .render();
                }
            },
        ],
        drawCallback: settings => {
            // const api = new $.fn.dataTable.Api(settings);
        }
    });
</script>
<?= $this->endSection() ?>
