import {Html} from './Html.js';

export class DataTableButtons {
    private _data: any;
    private _type: string = 'display';
    private _row: any;
    private _meta: any;
    private buttons: Array<any> = [];

    constructor(data: any, type: string, row: any, meta: any) {
        this.data = data;
        this.type = type;
        this.row = row;
        this.meta = meta;
    }

    public static init(data: any, type: string, row: any, meta: any): DataTableButtons {
        return new DataTableButtons(data, type, row, meta);
    }

    public add(label: string, url: string, icon: string | null = null, className: string | null = null): DataTableButtons {
        const i = Html.createElement('i', {class: icon}),
            a = Html.createElement('a', {class: `dropdown-item ${className}`, href: url}),
            li = Html.createElement('li');

        if (icon) {
            a.appendChild(i);
        }

        a.append(label);
        li.appendChild(a);

        this.buttons.push(li);

        return this;
    }

    public divider(): DataTableButtons {
        const hr = Html.createElement('hr', {class: 'dropdown-divider'}),
            li = Html.createElement('li', null, hr);
        this.buttons.push(li);
        return this;
    }

    public header(label: string): DataTableButtons {
        const h6 = Html.createElement('hr', {class: 'dropdown-header'}, label),
            li = Html.createElement('li', null, h6);
        this.buttons.push(li);
        return this;
    }

    public render(label: string | null = null) {
        const icon = Html.createElement('i', {class: "bi bi-three-dots"}),
            dropdownToggle = Html.createElement('button', {
                class: 'btn btn-sm btn-theme dropdown-toggle',
                type: 'button',
                'data-bs-toggle': 'dropdown',
                'aria-expanded': 'false'
            }, label ?? icon),
            dropdown = Html.createElement('div', {class: "dropdown"}, dropdownToggle),
            dropdownMenu = Html.createElement('ul', {class: "dropdown-menu"});
        this.buttons.forEach(button => dropdownMenu.appendChild(button));
        dropdown.appendChild(dropdownMenu);
        return dropdown.outerHTML;
    }

    get data(): any {
        return this._data;
    }

    set data(value: any) {
        this._data = value;
    }

    get type(): string {
        return this._type;
    }

    set type(value: string) {
        this._type = value;
    }

    get row(): any {
        return this._row;
    }

    set row(value: any) {
        this._row = value;
    }

    get meta(): any {
        return this._meta;
    }

    set meta(value: any) {
        this._meta = value;
    }
}